@@ -26,14 +26,14 @@ typedef struct xfs_log_item {
 
 #define XFS_LI_DIRTY	3	/* log item dirty in transaction */
 
-typedef struct xfs_inode_log_item {
+struct xfs_inode_log_item {
 	xfs_log_item_t		ili_item;		/* common portion */
 	struct xfs_inode	*ili_inode;		/* inode pointer */
 	unsigned short		ili_lock_flags;		/* lock flags */
 	unsigned int		ili_last_fields;	/* fields when flushed*/
 	unsigned int		ili_fields;		/* fields to be logged */
 	unsigned int		ili_fsync_fields;	/* ignored by userspace */
-} xfs_inode_log_item_t;
+};
 
 typedef struct xfs_buf_log_item {
 	xfs_log_item_t		bli_item;	/* common item structure */
@@ -47,6 +47,7 @@ typedef struct xfs_buf_log_item {
 #define XFS_BLI_HOLD			(1<<1)
 #define XFS_BLI_STALE			(1<<2)
 #define XFS_BLI_INODE_ALLOC_BUF		(1<<3)
+#define XFS_BLI_ORDERED			(1<<4)
 
 typedef struct xfs_qoff_logitem {
 	xfs_log_item_t		qql_item;	/* common portion */
