@@ -1223,49 +1223,17 @@ xfs_verify_magic16(
 kmem_zone_t		*xfs_inode_zone;
 extern kmem_zone_t	*xfs_ili_zone;
 
-/*
- * If there are inline format data / attr forks attached to this inode,
- * make sure they're not corrupt.
- */
-bool
-libxfs_inode_verify_forks(
-	struct xfs_inode	*ip)
-{
-	struct xfs_ifork	*ifp;
-	xfs_failaddr_t		fa;
-
-	if (!ip->i_fork_ops)
-		return true;
-
-	fa = xfs_ifork_verify_data(ip, ip->i_fork_ops);
-	if (fa) {
-		ifp = XFS_IFORK_PTR(ip, XFS_DATA_FORK);
-		xfs_inode_verifier_error(ip, -EFSCORRUPTED, "data fork",
-				ifp->if_u1.if_data, ifp->if_bytes, fa);
-		return false;
-	}
-
-	fa = xfs_ifork_verify_attr(ip, ip->i_fork_ops);
-	if (fa) {
-		ifp = XFS_IFORK_PTR(ip, XFS_ATTR_FORK);
-		xfs_inode_verifier_error(ip, -EFSCORRUPTED, "attr fork",
-				ifp ? ifp->if_u1.if_data : NULL,
-				ifp ? ifp->if_bytes : 0, fa);
-		return false;
-	}
-	return true;
-}
-
 int
 libxfs_iget(
 	struct xfs_mount	*mp,
 	struct xfs_trans	*tp,
 	xfs_ino_t		ino,
 	uint			lock_flags,
-	struct xfs_inode	**ipp,
-	struct xfs_ifork_ops	*ifork_ops)
+	struct xfs_inode	**ipp)
 {
 	struct xfs_inode	*ip;
+	struct xfs_dinode	*dip;
+	struct xfs_buf		*bp;
 	int			error = 0;
 
 	ip = kmem_zone_zalloc(xfs_inode_zone, 0);
@@ -1274,21 +1242,29 @@ libxfs_iget(
 
 	ip->i_ino = ino;
 	ip->i_mount = mp;
-	error = xfs_iread(mp, tp, ip, 0);
-	if (error) {
-		kmem_cache_free(xfs_inode_zone, ip);
-		*ipp = NULL;
-		return error;
-	}
+	error = xfs_imap(mp, tp, ip->i_ino, &ip->i_imap, 0);
+	if (error)
+		goto out_destroy;
 
-	ip->i_fork_ops = ifork_ops;
-	if (!libxfs_inode_verify_forks(ip)) {
-		libxfs_irele(ip);
-		return -EFSCORRUPTED;
-	}
+	error = xfs_imap_to_bp(mp, tp, &ip->i_imap, &dip, &bp, 0);
+	if (error)
+		goto out_destroy;
+
+	error = xfs_inode_from_disk(ip, dip);
+	if (!error)
+		xfs_buf_set_ref(bp, XFS_INO_REF);
+	xfs_trans_brelse(tp, bp);
+
+	if (error)
+		goto out_destroy;
 
 	*ipp = ip;
 	return 0;
+
+out_destroy:
+	kmem_cache_free(xfs_inode_zone, ip);
+	*ipp = NULL;
+	return error;
 }
 
 static void
@@ -1298,13 +1274,17 @@ libxfs_idestroy(xfs_inode_t *ip)
 		case S_IFREG:
 		case S_IFDIR:
 		case S_IFLNK:
-			libxfs_idestroy_fork(ip, XFS_DATA_FORK);
+			libxfs_idestroy_fork(&ip->i_df);
 			break;
 	}
-	if (ip->i_afp)
-		libxfs_idestroy_fork(ip, XFS_ATTR_FORK);
-	if (ip->i_cowfp)
-		xfs_idestroy_fork(ip, XFS_COW_FORK);
+	if (ip->i_afp) {
+		libxfs_idestroy_fork(ip->i_afp);
+		kmem_cache_free(xfs_ifork_zone, ip->i_afp);
+	}
+	if (ip->i_cowfp) {
+		libxfs_idestroy_fork(ip->i_cowfp);
+		kmem_cache_free(xfs_ifork_zone, ip->i_cowfp);
+	}
 }
 
 void
